@@ -16,7 +16,7 @@ use Carp;
 
 @ISA = qw(Exporter);
 @EXPORT = qw(pingecho);
-$VERSION = "2.31";
+$VERSION = "2.32";
 
 sub SOL_IP { 0; };
 sub IP_TOS { 1; };
@@ -35,11 +35,11 @@ $syn_forking = 0;
 if ($^O =~ /Win32/i) {
   # Hack to avoid this Win32 spewage:
   # Your vendor has not defined POSIX macro ECONNREFUSED
-  *ECONNREFUSED = sub {10061;}; # "Unknown Error" Special Win32 Response?
-  *ENOTCONN     = sub {10057;};
-  *ECONNRESET   = sub {10054;};
-  *EINPROGRESS  = sub {10036;};
-  *EWOULDBLOCK  = sub {10035;};
+  *ECONNREFUSED = sub() {10061;}; # "Unknown Error" Special Win32 Response?
+  *ENOTCONN     = sub() {10057;};
+  *ECONNRESET   = sub() {10054;};
+  *EINPROGRESS  = sub() {10036;};
+  *EWOULDBLOCK  = sub() {10035;};
 #  $syn_forking = 1;    # XXX possibly useful in < Win2K ?
 };
 
@@ -240,6 +240,7 @@ sub mselect
 	while (1) {
 	    $gran = $t if $gran > $t;
 	    my $nfound = select($_[0], $_[1], $_[2], $gran);
+	    undef $nfound if $nfound == -1;
 	    $t -= $gran;
 	    return $nfound if $nfound or !defined($nfound) or $t <= 0;
 
@@ -248,11 +249,13 @@ sub mselect
 	}
     }
     else {
-	return select($_[0], $_[1], $_[2], $_[3]);
+	my $nfound = select($_[0], $_[1], $_[2], $_[3]);
+	undef $nfound if $nfound == -1;
+	return $nfound;
     }
 }
 
-# Description: Allow UDP source endpoint comparision to be
+# Description: Allow UDP source endpoint comparison to be
 #              skipped for those remote interfaces that do
 #              not response from the same endpoint.
 
@@ -316,9 +319,9 @@ sub socket_blocking_mode
                         # set the non-blocking mode (set O_NONBLOCK)
 
   my $flags;
-  if ($^O eq 'MSWin32') {
-      # FIONBIO enables non-blocking sockets on windows.
-      # FIONBIO is (0x80000000|(4<<16)|(ord('f')<<8)|126), as per winsock.h.
+  if ($^O eq 'MSWin32' || $^O eq 'VMS') {
+      # FIONBIO enables non-blocking sockets on windows and vms.
+      # FIONBIO is (0x80000000|(4<<16)|(ord('f')<<8)|126), as per winsock.h, ioctl.h
       my $f = 0x8004667e;
       my $v = pack("L", $block ? 0 : 1);
       ioctl($fh, $f, $v) or croak("ioctl failed: $!");
@@ -395,12 +398,13 @@ sub ping_external {
   return Net::Ping::External::ping(ip => $ip, timeout => $timeout);
 }
 
-use constant ICMP_ECHOREPLY => 0; # ICMP packet types
-use constant ICMP_ECHO      => 8;
-use constant ICMP_STRUCT    => "C2 n3 A";  # Structure of a minimal ICMP packet
-use constant SUBCODE        => 0; # No ICMP subcode for ECHO and ECHOREPLY
-use constant ICMP_FLAGS     => 0; # No special flags for send or recv
-use constant ICMP_PORT      => 0; # No port with ICMP
+use constant ICMP_ECHOREPLY   => 0; # ICMP packet types
+use constant ICMP_UNREACHABLE => 3; # ICMP packet types
+use constant ICMP_ECHO        => 8;
+use constant ICMP_STRUCT      => "C2 n3 A"; # Structure of a minimal ICMP packet
+use constant SUBCODE          => 0; # No ICMP subcode for ECHO and ECHOREPLY
+use constant ICMP_FLAGS       => 0; # No special flags for send or recv
+use constant ICMP_PORT        => 0; # No port with ICMP
 
 sub ping_icmp
 {
@@ -478,10 +482,12 @@ sub ping_icmp
       $self->{"from_subcode"} = $from_subcode;
       if (($from_pid == $self->{"pid"}) && # Does the packet check out?
           ($from_seq == $self->{"seq"})) {
-        if ($from_type == ICMP_ECHOREPLY){
+        if ($from_type == ICMP_ECHOREPLY) {
           $ret = 1;
+	  $done = 1;
+        } elsif ($from_type == ICMP_UNREACHABLE) {
+          $done = 1;
         }
-        $done = 1;
       }
     } else {     # Oops, timed out
       $done = 1;
@@ -1471,7 +1477,7 @@ otherwise it will return false.  NOTE: Unlike the other protocols,
 the return value does NOT determine if the remote host is alive or
 not since the full TCP three-way handshake may not have completed
 yet.  The remote host is only considered reachable if it receives
-a TCP ACK within the timeout specifed.  To begin waiting for the
+a TCP ACK within the timeout specified.  To begin waiting for the
 ACK packets, use the ack() method as explained below.  Use the
 "syn" protocol instead the "tcp" protocol to determine reachability
 of multiple destinations simultaneously by sending parallel TCP
@@ -1503,10 +1509,10 @@ otherwise.  The maximum number of data bytes that can be specified is
 1024.
 
 If $device is given, this device is used to bind the source endpoint
-before sending the ping packet.  I beleive this only works with
+before sending the ping packet.  I believe this only works with
 superuser privileges and with udp and icmp protocols at this time.
 
-If $tos is given, this ToS is configured into the soscket.
+If $tos is given, this ToS is configured into the socket.
 
 =item $p->ping($host [, $timeout]);
 
@@ -1559,7 +1565,7 @@ This is disabled by default.
 
 =item $p->tcp_service_check( { 0 | 1 } );
 
-Depricated method, but does the same as service_check() method.
+Deprecated method, but does the same as service_check() method.
 
 =item $p->hires( { 0 | 1 } );
 
@@ -1608,7 +1614,7 @@ connection will not be established and ack() will return
 undef.  In list context, the host, the ack time, and the
 dotted ip string will be returned instead of just the host.
 If the optional $host argument is specified, the return
-value will be partaining to that host only.
+value will be pertaining to that host only.
 This call simply does nothing if you are using any protocol
 other than syn.
 
